Option Explicit

' ****************************************************************************
' *** VERSION ****************************************************************
' ****************************************************************************
'
' LaunchViaLabel.vbs
'  v1.00
'  14/Aug/2009
'  by Leo Davidson <leo@ox.compsoc.net> http://www.pretentiousname.com
'  Tested with Windows 7 RTM x64
'
'  v2.00
'  14/Aug/2009
'  Added the possibility to use file redirection for command line using  instead of >
'  Tested with Windows XP x86 SP3
'
'  v3.00
'  17/Aug/2009
'  Added the possibility to use "pause" argument to leave command interpreter window open for visual check
'  Tested with Windows XP x86 SP3

' ****************************************************************************
' *** PURPOSE ****************************************************************
' ****************************************************************************
' 
' This script runs a command on a particular drive, looking up the drive via
' its label. It can be useful when you want to launch something from a USB
' drive which doesn't always have the same label.
'

' ****************************************************************************
' *** USAGE ******************************************************************
' ****************************************************************************
'
' You should run the script with at least two arguments. The first argument is
' the label of the drive you want to look up. The rest of the arguments are
' the command you want to run.
'
' Any occurances of "__DRIVE__" in the arguments will be replaced with the
' drive letter (e.g. "X:").
'
'
' Example 1)
' If "My USB Stick" is mounted as U: then this:
'	LaunchViaLabel.vbs "My USB Stick" __DRIVE__\HelloWorld.exe
' Will run this:
'	U:\HelloWorld.exe
'
'
' Example 2)
' If "LEOCAMERA" is mounted as M: then this:
'	LaunchViaLabel.vbs "LEOCAMERA" Explorer.exe __DRIVE__
' Will run this:
'	Explorer.exe M:
'
' That will launch Explorer showing the M: drive.
'
' (If you want to respect the default file manager instead of launching
' Explorer explicitly, you'd just launch the drive letter by itself and let
' Windows decide what to run. In that case you should set WaitForCommand=False
' below. Otherwise you'll get an error message because the script can only
' wait for commands to complete if they run an executable directly.)
'
'
' Example 3)
' If there is the need to launch a command line with several parameters using redirection
' you should replace the ">" character by ""
'	LaunchViaLabel.vbs "My USB Stick" __DRIVE__\Command.exe -arg1 -arg2 Redir.log
' The Command.exe is launched using -arg1 & -arg2, and redirect the streams to Redir.log
'
'
' Example 4)
' If there is the need to visually check the output of the console, you can add "pause" argument
'	LaunchViaLabel.vbs "My USB Stick" __DRIVE__\Command.exe -arg1 -arg2 pause
' The Command.exe is launched using -arg1 & -arg2, then paused at the end of the command execution
'
' (This can work together with redirection even if there's no real need to see the output in the console
' when redirecting streams to a file.)


' ****************************************************************************
' *** CONFIG *****************************************************************
' ****************************************************************************
'
' Change WaitForCommand to False if you don't want the script to wait for the command to complete.

Dim WaitForCommand
WaitForCommand = False


' ****************************************************************************
' *** MAIN *******************************************************************
' ****************************************************************************

Dim Args
Set Args = WScript.Arguments

Dim Shell
Set Shell = CreateObject("WScript.Shell")

Dim WmiCim
Set WmiCim = GetObject("winmgmts:\\.\root\cimv2")

If Args.Count < 2 Then
	WScript.Echo Wscript.ScriptName & ": Run with at least two arguments. See comments at the top of the script for instructions."
	WScript.Quit 1
End If

Dim DriveLabel
DriveLabel = Args(0)

Dim DriveList
Set DriveList = WmiCim.ExecQuery("Select * from Win32_LogicalDisk where VolumeName='" & DriveLabel & "'")

If DriveList.Count = 0 Then
	WScript.Echo Wscript.ScriptName & ": No drive with label """ & DriveLabel & """ could not be found."
	WScript.Quit 1
End If

If DriveList.Count > 1 Then
	WScript.Echo Wscript.ScriptName & ": Multiple drives with label """ & DriveLabel & """ were found."
	WScript.Quit 1
End If

Dim Drive
Dim i
Dim Quote
Dim CmdLine
Dim Redir
Dim Pause
Pause = False
CmdLine = ""
Redir = ""

For i = 1 To Args.Count - 1
	If InStr(Args(i), "") Then
		Redir = Args(i)
		Redir = Replace(Redir, "", "")
	Else
		If InStr(Args(i), "pause") Then
			Pause = True
		Else
			If i > 1 Then
				CmdLine = CmdLine & " "
			End If

			If InStr(Args(i), " ") <> 0 Then
				Quote = """"
			Else
				Quote = ""
			End If

			CmdLine = CmdLine & Quote & Args(i) & Quote
		End If
	End If
Next


For Each Drive In DriveList
	CmdLine = Replace(CmdLine, "__DRIVE__", Drive.Name)
Next


If InStr(Redir, "") <> 0 Then
	If Pause = True Then
		Shell.Run "%ComSpec% /c" & CmdLine & " > " & Redir & "& pause", 1, WaitForCommand
	Else
		Shell.Run "%ComSpec% /c" & CmdLine & " > " & Redir, 1, WaitForCommand
	End If
Else
	If Pause = True Then
		Shell.Run "%ComSpec% /c" & CmdLine & "& pause", 1, WaitForCommand
	Else
		Shell.Run CmdLine, 1, WaitForCommand
	End If
End If
